from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django.utils.translation import ugettext_lazy as _

from name.models import (
    Name,
    Variant,
    Note,
    Identifier,
    Identifier_Type,
    Location
)


# we need this custom filter for the boolean sidebar merged_with toggle
class IsMergedWithFilter(SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _('merged status')
    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'merged_with'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each tuple is the coded
        value for the option that will appear in the URL query. The second
        element is the human-readable name for the option that will appear
        in the right sidebar.
        """

        return (
            ('Yes', _('Currently merged')),
            ('No', _('Not merged')),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """

        # Compare the requested value (either 'Yes' or 'No')
        # to decide how to filter the queryset.
        if self.value() == 'Yes':
            # the merged_with foreign key will be None if not merged
            return queryset.exclude(merged_with=None)
        if self.value() == 'No':
            return queryset.filter(merged_with=None)


class VariantInline(admin.TabularInline):
    model = Variant
    extra = 0


class MergedWithInline(admin.TabularInline):

    # we dont want to add any records to this inline, just show merged
    def has_add_permission(self, request):
        return False

    model = Name
    verbose_name_plural = 'names that are merged with this record'
    verbose_name = 'names merged with this record'
    readonly_fields = ('name',)
    fields = ('name',)
    extra = 0
    can_delete = False


class IdentifierInline(admin.TabularInline):
    model = Identifier
    extra = 0


class NoteInline(admin.TabularInline):
    model = Note
    extra = 0


class Identifier_TypeAdmin(admin.ModelAdmin):
    model = Identifier_Type
    extra = 0


class LocationInline(admin.StackedInline):
    model = Location
    extra = 0


class NameAdmin(admin.ModelAdmin):

    # what to display/order of Names list /admin/name/name
    list_display = [
        'name',
        'name_id',
        'disambiguation',
        'begin',
        'end',
        'record_status',
        'has_geocode',
    ]

    # list of fields that the search queries
    search_fields = [
        'name',
        'biography',
        'variant__variant',
        'disambiguation',
    ]

    raw_id_fields = [
        'merged_with'
    ]

    # right-side filter toggle
    list_filter = [
        'record_status',
        'name_type',
        IsMergedWithFilter,
    ]

    # non-editable
    readonly_fields = [
        'name_id',
        'normalized_name',
    ]

    # FK inline fields
    inlines = [
        MergedWithInline,
        VariantInline,
        IdentifierInline,
        NoteInline,
        LocationInline,
    ]
    no_merged_inlines = [
        VariantInline,
        IdentifierInline,
        NoteInline,
        LocationInline,
    ]

    # these functions decide which set of inlines should be displayed
    def get_inline_instances(self, request, obj=None):
        inline_instances = []

        inlines = self.no_merged_inlines
        for inline_class in inlines:
            inline = inline_class(self.model, self.admin_site)
            if request:
                if not (inline.has_add_permission(request) or
                        inline.has_change_permission(request) or
                        inline.has_delete_permission(request)):
                    continue
                if not inline.has_add_permission(request):
                    inline.max_num = 0
            inline_instances.append(inline)
        return inline_instances

    def get_formsets(self, request, obj=None):
        for inline in self.get_inline_instances(request, obj):
            yield inline.get_formset(request, obj)

    # edit page fieldset / order
    fieldsets = [
        (None, {
            'fields': [
                'name_id',
            ]
        }),
        ('Basic Info', {
            'fields': [
                'name',
                'normalized_name',
                'name_type',
                'biography',
                'begin',
                'end',
                'disambiguation',
            ]
        }),
        ('Misc Options', {
            'fields': [
                'record_status',
                'merged_with',
            ]
        }),
    ]

admin.site.register(Name, NameAdmin)
admin.site.register(Identifier_Type, Identifier_TypeAdmin)



from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse_lazy
from django.utils.feedgenerator import Atom1Feed

from . import app_settings
from .models import Name


class NameAtomFeedType(Atom1Feed):
    """Custom Atom Feed Type for Name objects.

    This will set the Content-Type header to application/xml, and
    add a georss:point element to any entry that has a current location
    set.
    """
    mime_type = 'application/xml'
    GEO_RSS_NS = 'http://www.georss.org/georss'

    def root_attributes(self):
        attrs = super(NameAtomFeedType, self).root_attributes()
        attrs['xmlns:georss'] = self.GEO_RSS_NS
        return attrs

    def add_item_elements(self, handler, item):
        super(NameAtomFeedType, self).add_item_elements(handler, item)

        # Add the georss:point element if the item has a geo_point
        # attribute.
        geo_point = item.get('geo_point', None)
        if geo_point:
            handler.addQuickElement('georss:point', geo_point)


class NameAtomFeed(Feed):
    """Atom Feed for Name objects."""
    feed_type = NameAtomFeedType
    link = reverse_lazy("name:feed")
    title = app_settings.NAME_APP_TITLE
    subtitle = "New Name Records"

    author_name = app_settings.NAME_FEED_AUTHOR_NAME
    author_email = app_settings.NAME_FEED_AUTHOR_EMAIL
    author_link = app_settings.NAME_FEED_AUTHOR_LINK

    def items(self):
        return Name.objects.order_by('-date_created')[:20]

    def item_title(self, obj):
        return obj.name

    def item_description(self, obj):
        return 'Name Type: {0}'.format(obj.get_name_type_label())

    def item_link(self, obj):
        return obj.get_absolute_url()

    def item_updateddate(self, obj):
        return obj.last_modified

    def item_location(self, obj):
        if obj.has_current_location():
            return obj.location_set.current_location.geo_point()

    def item_extra_kwargs(self, obj):
        return {u'geo_point': self.item_location(obj)}



from django.core.exceptions import ValidationError


def follow_merged_with(name):
    """A generator to get the merged_with relationship
    of a Name object.

    This will return a Name object until it reaches a Name that
    does not have a merged_with relationship.
    """
    while name:
        merged_into = name.merged_with
        if merged_into:
            yield merged_into
        name = merged_into


def validate_merged_with(name):
    """Validator for the merged_with ForeignKey field.

    This will prevent two scenarios from occurring.
    1. Merging with a nonexistent Name object.

    2. Creating a loop of foreign key relationships.
        For example:
            Name 1 -> Name 2 -> Name 3 -> Name 1

        We need to prevent this because navigating to a name that has
        been merged with another, will redirect you to the Name it has
        been merged with.  If a loop is created, we will also create
        the opportunity for an HTTP redirect loop.

    Unlike typical Django validators, this requires a model instance
    as a parameter, instead of the value, which in this case would have
    been the ID of the related model.  Because of this requirement, this
    validator cannot be added via the `validator` kwarg on a ForeignKey
    field.  Rather this method should be called from the `clean` method.
    """
    # Return early if there is no need to validate.
    if name.merged_with is None:
        return

    # Get the Name class from the model instance, to avoid
    # circular importing name.models.Name.
    Name = name.__class__

    # Prevent the user from attempting to merge with a nonexistent
    # Name.
    try:
        merge_target = Name.objects.get(id=name.merged_with_id)
    except Name.DoesNotExist:
        raise ValidationError(
            dict(merged_with=u'The merge target must exist.'))

    # Prevent the user from attempting to merge a name with itself.
    if name.merged_with_id == name.id:
        raise ValidationError(
            dict(merged_with=u'Unable to merge a Name with itself.'))

    # Iterate through the generator and keep track of the return names.
    # We will find a loop if the return name is already in
    # merged_list. If this happens we will raise a validation error.
    # If we don't find duplicates, then no loop has been created and
    # the generator will raise it's own StopIteration and we will
    # implicitly return.
    merge_sequence = [name]
    for name in follow_merged_with(merge_target):
        if name in merge_sequence:
            msg = (u'The specified merge action completes a merge loop. '
                   'Unable to complete merge.')
            raise ValidationError(dict(merged_with=msg))
        merge_sequence.append(name)



from .models import Name
from . import app_settings


def name(request):
    """Adds the Name Types and branding options to the request
    context.
    """
    return {
        'name_types': dict(Name.NAME_TYPE_CHOICES),
        'name_app_title': app_settings.NAME_APP_TITLE,
        'name_admin_email': app_settings.NAME_ADMIN_EMAIL
    }



import csv

from django import http
from django.views import generic
from django.core.urlresolvers import reverse
from django.templatetags.static import static
from django.shortcuts import get_object_or_404, render, redirect
from pynaco.naco import normalizeSimplified

from .models import Name, Identifier
from .utils import filter_names


def label(request, name_value):
    """Find a name by label.

    If the label matches a single name, users are redirected
    to the name detail page. Otherwise, if the label
    does not exist, or the specified label matches multiple Name
    objects, a status code 404 is returned.
    """
    if not name_value:
        return http.HttpResponseNotFound()

    normalized_name = normalizeSimplified(name_value)
    try:
        name = Name.objects.get(normalized_name=normalized_name)

        return http.HttpResponseRedirect(
            reverse('name:detail', args=[name.name_id]))

    except Name.DoesNotExist:
        return http.HttpResponseNotFound(
            'No matching term found - authoritative or variant - for "{0}"'
            .format(name_value))

    except Name.MultipleObjectsReturned:
        return http.HttpResponseNotFound(
            'There are multiple Name objects with the same name: "{0}".'
            .format(normalized_name))


def detail(request, name_id):
    """View for the Name detail page."""
    queryset = (
        Name.objects.select_related().
        prefetch_related('identifier_set__type')
    )

    name_entry = get_object_or_404(queryset, name_id=name_id)

    if name_entry.merged_with:
        return redirect(name_entry.merged_with)

    elif name_entry.is_suppressed():
        return http.HttpResponseNotFound(
            'The requested record could not be found.')

    elif name_entry.is_deleted():
        return http.HttpResponseGone('The requested record has been deleted!')

    return render(request, 'name/name_detail.html', dict(name=name_entry))


def export(request):
    """Exports Names to a TSV file."""
    # Create a CSV response.
    response = http.HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="data.tsv"'

    writer = csv.writer(response, delimiter='\t', quoting=csv.QUOTE_MINIMAL)

    # Iterate through the visible Names that are not merged with any other
    # Name records and write a row for each record.
    for n in Name.objects.visible():
        writer.writerow([
            n.get_name_type_label().lower(),
            n.name.encode('utf-8'),
            request.build_absolute_uri(n.get_absolute_url())
        ])

    return response


def opensearch(request):
    """Renders an Opensearch XML file.

    Opensearch URL templates are composed in the view
    to more easily compose the absolute url with the
    query string.
    """
    # Query templates for the URLs.
    search_query = '{0}?q_type=Personal&q={{searchTerms}}'
    search_json_query = '{0}?q={{searchTerms}}'

    # Build the absolute URLs.
    search_url = request.build_absolute_uri(reverse('name:search'))
    search_json_url = request.build_absolute_uri(reverse('name:search-json'))

    # Compose the full URLs that will be sent to the template.
    search = search_query.format(search_url)
    search_json = search_json_query.format(search_json_url)
    image = request.build_absolute_uri(static('name/img/favicon.png'))

    urls = dict(search=search, search_json=search_json, image=image)

    return render(request, 'name/opensearch.xml',
                  urls, content_type='text/xml')


def about(request):
    """View for the About page."""
    return render(request, 'name/about.html')


def stats(request):
    """View for the Stats page."""
    context = dict(
        total_names=Name.objects.visible().count(),
        total_identifiers=Identifier.objects.count()
    )
    return render(request, 'name/stats.html', context)


def landing(request):
    """View for the Landing page."""
    counts = dict(counts=Name.objects.active_type_counts())
    return render(request, 'name/landing.html', counts)


def locations(request):
    """View for the Map page."""
    return render(request, 'name/map.html')


class SearchView(generic.ListView):
    model = Name
    template_name = 'name/search.html'
    paginate_by = 15

    VALID_SORTS = {
        'name_a': 'name',
        'name_d': '-name',
        'begin_a': 'begin',
        'begin_d': '-begin',
        'end_a': 'end',
        'end_d': '-end',
    }

    DEFAULT_SORT = VALID_SORTS['name_a']

    def get_sort_method(self):
        order = self.request.GET.get('order', '')
        return self.VALID_SORTS.get(order, self.DEFAULT_SORT)

    def get_queryset(self):
        query = self.request.GET
        if query.get('q') or query.get('q_type'):
            return filter_names(self.request).order_by(self.get_sort_method())
        return Name.objects.none()


def mads_serialize(request, name_id):
    """Renders the Name serialized into the MADS XML format."""
    queryset = (
        Name.objects.select_related()
        .prefetch_related('identifier_set__type', 'note_set', 'variant_set')
    )

    context = dict(name=get_object_or_404(queryset, name_id=name_id))

    return render(request, 'name/name.mads.xml',
                  context, content_type='text/xml')



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Identifier_Type'
        db.create_table(u'name_identifier_type', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('icon_path', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('homepage', self.gf('django.db.models.fields.URLField')(max_length=200, blank=True)),
        ))
        db.send_create_signal(u'name', ['Identifier_Type'])

        # Adding model 'Identifier'
        db.create_table(u'name_identifier', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['name.Identifier_Type'])),
            ('belong_to_name', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['name.Name'])),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=500)),
            ('visible', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')(default=0)),
        ))
        db.send_create_signal(u'name', ['Identifier'])

        # Adding model 'Note'
        db.create_table(u'name_note', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('note', self.gf('django.db.models.fields.TextField')()),
            ('note_type', self.gf('django.db.models.fields.IntegerField')()),
            ('belong_to_name', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['name.Name'])),
        ))
        db.send_create_signal(u'name', ['Note'])

        # Adding model 'Variant'
        db.create_table(u'name_variant', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('belong_to_name', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['name.Name'])),
            ('variant_type', self.gf('django.db.models.fields.IntegerField')(max_length=50)),
            ('variant', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('normalized_variant', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'name', ['Variant'])

        # Adding model 'BaseTicketing'
        db.create_table(u'name_baseticketing', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('stub', self.gf('django.db.models.fields.BooleanField')(default=True, unique=True)),
        ))
        db.send_create_signal(u'name', ['BaseTicketing'])

        # Adding model 'Name'
        db.create_table(u'name_name', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('normalized_name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('name_type', self.gf('django.db.models.fields.IntegerField')(max_length=1)),
            ('begin', self.gf('django.db.models.fields.CharField')(max_length=25, blank=True)),
            ('end', self.gf('django.db.models.fields.CharField')(max_length=25, blank=True)),
            ('disambiguation', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('biography', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('record_status', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('merged_with', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='merged_with_name', null=True, to=orm['name.Name'])),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('last_modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('name_id', self.gf('django.db.models.fields.CharField')(unique=True, max_length=10)),
        ))
        db.send_create_signal(u'name', ['Name'])

        # Adding unique constraint on 'Name', fields ['name', 'name_id']
        db.create_unique(u'name_name', ['name', 'name_id'])

        # Adding model 'Location'
        db.create_table(u'name_location', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('belong_to_name', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['name.Name'])),
            ('latitude', self.gf('django.db.models.fields.DecimalField')(max_digits=13, decimal_places=10)),
            ('longitude', self.gf('django.db.models.fields.DecimalField')(max_digits=13, decimal_places=10)),
            ('status', self.gf('django.db.models.fields.IntegerField')(default=0, max_length=2)),
        ))
        db.send_create_signal(u'name', ['Location'])


    def backwards(self, orm):
        # Removing unique constraint on 'Name', fields ['name', 'name_id']
        db.delete_unique(u'name_name', ['name', 'name_id'])

        # Deleting model 'Identifier_Type'
        db.delete_table(u'name_identifier_type')

        # Deleting model 'Identifier'
        db.delete_table(u'name_identifier')

        # Deleting model 'Note'
        db.delete_table(u'name_note')

        # Deleting model 'Variant'
        db.delete_table(u'name_variant')

        # Deleting model 'BaseTicketing'
        db.delete_table(u'name_baseticketing')

        # Deleting model 'Name'
        db.delete_table(u'name_name')

        # Deleting model 'Location'
        db.delete_table(u'name_location')


    models = {
        u'name.baseticketing': {
            'Meta': {'object_name': 'BaseTicketing'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'stub': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'unique': 'True'})
        },
        u'name.identifier': {
            'Meta': {'ordering': "['order', 'type']", 'object_name': 'Identifier'},
            'belong_to_name': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['name.Name']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['name.Identifier_Type']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '500'}),
            'visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'name.identifier_type': {
            'Meta': {'ordering': "['label']", 'object_name': 'Identifier_Type'},
            'homepage': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'icon_path': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'name.location': {
            'Meta': {'ordering': "['status']", 'object_name': 'Location'},
            'belong_to_name': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['name.Name']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'latitude': ('django.db.models.fields.DecimalField', [], {'max_digits': '13', 'decimal_places': '10'}),
            'longitude': ('django.db.models.fields.DecimalField', [], {'max_digits': '13', 'decimal_places': '10'}),
            'status': ('django.db.models.fields.IntegerField', [], {'default': '0', 'max_length': '2'})
        },
        u'name.name': {
            'Meta': {'ordering': "['name']", 'unique_together': "(('name', 'name_id'),)", 'object_name': 'Name'},
            'begin': ('django.db.models.fields.CharField', [], {'max_length': '25', 'blank': 'True'}),
            'biography': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'disambiguation': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'end': ('django.db.models.fields.CharField', [], {'max_length': '25', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'merged_with': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'merged_with_name'", 'null': 'True', 'to': u"orm['name.Name']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'name_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '10'}),
            'name_type': ('django.db.models.fields.IntegerField', [], {'max_length': '1'}),
            'normalized_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'record_status': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        u'name.note': {
            'Meta': {'object_name': 'Note'},
            'belong_to_name': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['name.Name']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'note': ('django.db.models.fields.TextField', [], {}),
            'note_type': ('django.db.models.fields.IntegerField', [], {})
        },
        u'name.variant': {
            'Meta': {'object_name': 'Variant'},
            'belong_to_name': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['name.Name']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'normalized_variant': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'variant': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'variant_type': ('django.db.models.fields.IntegerField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['name']


from ..models import Name
from django.utils import timezone
from dateutil.relativedelta import relativedelta


class NameStatisticsMonth(object):
    """A simple datatype to represent Name statistics for a
    single month.
    """

    def __init__(self, **kwargs):
        self.total = kwargs.get('total', 0)
        self.total_to_date = kwargs.get('total_to_date', 0)
        self.month = kwargs.get('month', None)


class NameStatisticsType(object):
    """Statistics class for calculating the number
    of Name objects in the database using a DateTime field.

    Accepts a ValuesQuerySet of dictionaries in the form of
        [{ count: <num>, month: <datetime object> }, ...]

    This will calculate:
        1. The overall total of of Name objects according to the
           the Iterable passed in.
        2. Starting from the first Name created, this will create
           a new NameStatisticsMonth object for each month up to
           the current month according to the system time.
    """

    def __init__(self, queryset):
        self.running_total = 0
        self.queryset = queryset
        self.stats = []

    def calculate(self):
        """Calculate the running total of the Name objects
        and total and total_to_date for each month since the
        first Name was created.
        """
        # Reset stats and running total to ensure they are not
        # mutated unintentionally if the method is executed successively.
        self.stats = []
        self.running_total = 0

        # Create a NameStatisticsMonth object for each month since the
        # first Name was created.
        for u in self.get_queryset_members():
            stats_month = NameStatisticsMonth(
                total=u.get('count'), month=u.get('month'))

            # Update the running total and use that value to
            # set the NameStatisticsMonth total_to_date property.
            self.running_total += u.get('count', 0)
            stats_month.total_to_date = self.running_total

            self.stats.append(stats_month)
        return self.stats

    def get_queryset_members(self):
        """Produces a generator which yields each element in the queryset
        in order.

        The queryset used to initialize this object is expected to
        only contain elements for each month where a name was created
        or modified (based on the queryset). If there is a month when a
        Name was not created, this method will instead create an element
        with `count` set to 0 for said month and yield it.
        """
        # Return early if the queryset is empty.
        if not len(self.queryset) > 0:
            return

        # Use the month in the first element of the queryset
        # as the starting month.
        current = self.queryset.first().get('month')

        # Convert the queryset to a list so that we can use the
        # pop() method.
        queryset = list(self.queryset)

        # Create a datetime object for the first day of the current
        # month according to the system time.
        end = timezone.now().replace(day=1, hour=0, minute=0, second=0,
                                     microsecond=0)

        # Set up the delta to increment the `current` date in the generator.
        delta = relativedelta(months=1)

        while current <= end:
            # Get the first element of the list, if it exists.
            elem = queryset[0] if len(queryset) else False

            # Yield the first queryset element if the element
            # exists and if it's member month is equal to `current`.
            if elem and elem.get('month') == current:
                yield queryset.pop(0)
            else:
                yield dict(count=0, month=current)
            current += delta


class NameStatistics(object):
    """Container class for all statistics gathered on
    Name objects.
    """

    def __init__(self):
        self.created = NameStatisticsType(Name.objects.created_stats())
        self.modified = NameStatisticsType(Name.objects.modified_stats())
        self.name_type_totals = Name.objects.active_type_counts()
        self.calculate()

    def calculate(self):
        self.created.calculate()
        self.modified.calculate()
